@@ -27,6 +27,7 @@ import oslo_messaging as messaging
 from oslo_service import service
 from oslo_utils import importutils
 
+from nova.api import wsgi as api_wsgi
 from nova import baserpc
 from nova import conductor
 import nova.conf
@@ -96,7 +97,7 @@ def setup_profiler(binary, host):
 
 
 def assert_eventlet_uses_monotonic_clock():
-    import eventlet.hubs as hubs
+    from eventlet import hubs
     import monotonic
 
     hub = hubs.get_hub()
@@ -152,6 +153,20 @@ class Service(service.Service):
         This includes starting an RPC service, initializing
         periodic tasks, etc.
         """
+        # NOTE(melwitt): Clear the cell cache holding database transaction
+        # context manager objects. We do this to ensure we create new internal
+        # oslo.db locks to avoid a situation where a child process receives an
+        # already locked oslo.db lock when it is forked. When a child process
+        # inherits a locked oslo.db lock, database accesses through that
+        # transaction context manager will never be able to acquire the lock
+        # and requests will fail with CellTimeout errors.
+        # See https://bugs.python.org/issue6721 for more information.
+        # With python 3.7, it would be possible for oslo.db to make use of the
+        # os.register_at_fork() method to reinitialize its lock. Until we
+        # require python 3.7 as a mininum version, we must handle the situation
+        # outside of oslo.db.
+        context.CELL_CACHE = {}
+
         assert_eventlet_uses_monotonic_clock()
 
         verstr = version.version_string_with_package()
@@ -307,6 +322,8 @@ class Service(service.Service):
     def reset(self):
         """reset the service."""
         self.manager.reset()
+        # Reset the cell cache that holds database transaction context managers
+        context.CELL_CACHE = {}
 
 
 class WSGIService(service.Service):
@@ -321,12 +338,16 @@ class WSGIService(service.Service):
 
         """
         self.name = name
-        # NOTE(danms): Name can be metadata, osapi_compute, or ec2, per
+        # NOTE(danms): Name can be metadata, osapi_compute, per
         # nova.service's enabled_apis
         self.binary = 'nova-%s' % name
+
+        LOG.warning('Running %s using eventlet is deprecated. Deploy with '
+                    'a WSGI server such as uwsgi or mod_wsgi.', self.binary)
+
         self.topic = None
         self.manager = self._get_manager()
-        self.loader = loader or wsgi.Loader()
+        self.loader = loader or api_wsgi.Loader()
         self.app = self.loader.load_app(name)
         # inherit all compute_api worker counts from osapi_compute
         if name.startswith('openstack_compute_api'):
@@ -357,13 +378,18 @@ class WSGIService(service.Service):
         setup_profiler(name, self.host)
 
     def reset(self):
-        """Reset server greenpool size to default and service version cache.
+        """Reset the following:
+
+        * server greenpool size to default
+        * service version cache
+        * cell cache holding database transaction context managers
 
         :returns: None
 
         """
         self.server.reset()
         service_obj.Service.clear_min_version_cache()
+        context.CELL_CACHE = {}
 
     def _get_manager(self):
         """Initialize a Manager object appropriate for this service.
@@ -391,6 +417,20 @@ class WSGIService(service.Service):
         :returns: None
 
         """
+        # NOTE(melwitt): Clear the cell cache holding database transaction
+        # context manager objects. We do this to ensure we create new internal
+        # oslo.db locks to avoid a situation where a child process receives an
+        # already locked oslo.db lock when it is forked. When a child process
+        # inherits a locked oslo.db lock, database accesses through that
+        # transaction context manager will never be able to acquire the lock
+        # and requests will fail with CellTimeout errors.
+        # See https://bugs.python.org/issue6721 for more information.
+        # With python 3.7, it would be possible for oslo.db to make use of the
+        # os.register_at_fork() method to reinitialize its lock. Until we
+        # require python 3.7 as a mininum version, we must handle the situation
+        # outside of oslo.db.
+        context.CELL_CACHE = {}
+
         ctxt = context.get_admin_context()
         service_ref = objects.Service.get_by_host_and_binary(ctxt, self.host,
                                                              self.binary)
