@@ -15,13 +15,13 @@
 #    under the License.
 
 from oslo_config import cfg
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import exception
 from nova.network import model
 from nova import objects
 from nova import test
 from nova.tests.unit import fake_network_cache_model
-from nova.tests import uuidsentinel as uuids
 from nova.virt import netutils
 
 
