@@ -104,14 +104,20 @@ class HostManagerTestCase(test.NoDBTestCase):
                                             transport_url='fake:///mq3',
                                             disabled=False)
         cells = [cell_mapping1, cell_mapping2, cell_mapping3]
+        # Throw a random host-to-cell in that cache to make sure it gets reset.
+        self.host_manager.host_to_cell_uuid['fake-host'] = cell_uuid1
         with mock.patch('nova.objects.CellMappingList.get_all',
                         return_value=cells) as mock_cm:
             self.host_manager.refresh_cells_caches()
             mock_cm.assert_called_once()
+        # Cell2 is not in the enabled list.
         self.assertEqual(2, len(self.host_manager.enabled_cells))
         self.assertEqual(cell_uuid3, self.host_manager.enabled_cells[1].uuid)
+        # But it is still in the full list.
         self.assertEqual(3, len(self.host_manager.cells))
-        self.assertEqual(cell_uuid2, self.host_manager.cells[1].uuid)
+        self.assertIn(cell_uuid2, self.host_manager.cells)
+        # The host_to_cell_uuid cache should have been reset.
+        self.assertEqual({}, self.host_manager.host_to_cell_uuid)
 
     def test_refresh_cells_caches_except_cell0(self):
         ctxt = nova_context.RequestContext('fake-user', 'fake_project')
@@ -129,6 +135,46 @@ class HostManagerTestCase(test.NoDBTestCase):
             mock_cm.assert_called_once()
         self.assertEqual(0, len(self.host_manager.cells))
 
+    @mock.patch('nova.objects.HostMapping.get_by_host',
+                return_value=objects.HostMapping(
+                    cell_mapping=objects.CellMapping(uuid=uuids.cell1)))
+    def test_get_cell_mapping_for_host(self, mock_get_by_host):
+        # Starting with an empty cache, assert that the HostMapping is looked
+        # up and the result is cached.
+        ctxt = nova_context.get_admin_context()
+        host = 'fake-host'
+        self.assertEqual({}, self.host_manager.host_to_cell_uuid)
+        cm = self.host_manager._get_cell_mapping_for_host(ctxt, host)
+        self.assertIs(cm, mock_get_by_host.return_value.cell_mapping)
+        self.assertIn(host, self.host_manager.host_to_cell_uuid)
+        self.assertEqual(
+            uuids.cell1, self.host_manager.host_to_cell_uuid[host])
+        mock_get_by_host.assert_called_once_with(ctxt, host)
+
+        # Reset the mock and do it again, assert we do not query the DB.
+        mock_get_by_host.reset_mock()
+        self.host_manager._get_cell_mapping_for_host(ctxt, host)
+        mock_get_by_host.assert_not_called()
+
+        # Mix up the cache such that the host is mapped to a cell that
+        # is not in the cache which will make us query the DB. Also make the
+        # HostMapping query raise HostMappingNotFound to make sure that comes
+        # up to the caller.
+        mock_get_by_host.reset_mock()
+        self.host_manager.host_to_cell_uuid[host] = uuids.random_cell
+        mock_get_by_host.side_effect = exception.HostMappingNotFound(name=host)
+        with mock.patch('nova.scheduler.host_manager.LOG.warning') as warning:
+            self.assertRaises(exception.HostMappingNotFound,
+                              self.host_manager._get_cell_mapping_for_host,
+                              ctxt, host)
+            # We should have logged a warning because the host is cached to
+            # a cell uuid but that cell uuid is not in the cells cache.
+            warning.assert_called_once()
+            self.assertIn('Host %s is expected to be in cell %s',
+                          warning.call_args[0][0])
+            # And we should have also tried to lookup the HostMapping in the DB
+            mock_get_by_host.assert_called_once_with(ctxt, host)
+
     @mock.patch.object(nova.objects.InstanceList, 'get_by_filters')
     @mock.patch.object(nova.objects.ComputeNodeList, 'get_all')
     def test_init_instance_info_batches(self, mock_get_all,
@@ -539,16 +585,19 @@ class HostManagerTestCase(test.NoDBTestCase):
     @mock.patch('nova.objects.ServiceList.get_by_binary')
     @mock.patch('nova.objects.ComputeNodeList.get_all')
     @mock.patch('nova.objects.InstanceList.get_uuids_by_host')
-    def test_get_all_host_states(self, mock_get_by_host, mock_get_all,
-                                 mock_get_by_binary, mock_log):
+    def test_get_host_states(self, mock_get_by_host, mock_get_all,
+                             mock_get_by_binary, mock_log):
         mock_get_by_host.return_value = []
         mock_get_all.return_value = fakes.COMPUTE_NODES
         mock_get_by_binary.return_value = fakes.SERVICES
         context = 'fake_context'
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
 
-        # get_all_host_states returns a generator, so make a map from it
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
-                           self.host_manager.get_all_host_states(context)}
+                           self.host_manager._get_host_states(
+                               context, compute_nodes, services)}
         self.assertEqual(4, len(host_states_map))
 
         calls = [
@@ -598,17 +647,22 @@ class HostManagerTestCase(test.NoDBTestCase):
     @mock.patch.object(host_manager.HostState, '_update_from_compute_node')
     @mock.patch.object(objects.ComputeNodeList, 'get_all')
     @mock.patch.object(objects.ServiceList, 'get_by_binary')
-    def test_get_all_host_states_with_no_aggs(self, svc_get_by_binary,
-                                              cn_get_all, update_from_cn,
-                                              mock_get_by_host):
+    def test_get_host_states_with_no_aggs(self, svc_get_by_binary,
+                                          cn_get_all, update_from_cn,
+                                          mock_get_by_host):
         svc_get_by_binary.return_value = [objects.Service(host='fake')]
         cn_get_all.return_value = [
             objects.ComputeNode(host='fake', hypervisor_hostname='fake')]
         mock_get_by_host.return_value = []
         self.host_manager.host_aggregates_map = collections.defaultdict(set)
 
-        hosts = self.host_manager.get_all_host_states('fake-context')
-        # get_all_host_states returns a generator, so make a map from it
+        context = nova_context.get_admin_context()
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         host_state = host_states_map[('fake', 'fake')]
@@ -618,10 +672,10 @@ class HostManagerTestCase(test.NoDBTestCase):
     @mock.patch.object(host_manager.HostState, '_update_from_compute_node')
     @mock.patch.object(objects.ComputeNodeList, 'get_all')
     @mock.patch.object(objects.ServiceList, 'get_by_binary')
-    def test_get_all_host_states_with_matching_aggs(self, svc_get_by_binary,
-                                                    cn_get_all,
-                                                    update_from_cn,
-                                                    mock_get_by_host):
+    def test_get_host_states_with_matching_aggs(self, svc_get_by_binary,
+                                                cn_get_all,
+                                                update_from_cn,
+                                                mock_get_by_host):
         svc_get_by_binary.return_value = [objects.Service(host='fake')]
         cn_get_all.return_value = [
             objects.ComputeNode(host='fake', hypervisor_hostname='fake')]
@@ -631,8 +685,13 @@ class HostManagerTestCase(test.NoDBTestCase):
             set, {'fake': set([1])})
         self.host_manager.aggs_by_id = {1: fake_agg}
 
-        hosts = self.host_manager.get_all_host_states('fake-context')
-        # get_all_host_states returns a generator, so make a map from it
+        context = nova_context.get_admin_context()
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         host_state = host_states_map[('fake', 'fake')]
@@ -642,11 +701,10 @@ class HostManagerTestCase(test.NoDBTestCase):
     @mock.patch.object(host_manager.HostState, '_update_from_compute_node')
     @mock.patch.object(objects.ComputeNodeList, 'get_all')
     @mock.patch.object(objects.ServiceList, 'get_by_binary')
-    def test_get_all_host_states_with_not_matching_aggs(self,
-                                                        svc_get_by_binary,
-                                                        cn_get_all,
-                                                        update_from_cn,
-                                                        mock_get_by_host):
+    def test_get_host_states_with_not_matching_aggs(self, svc_get_by_binary,
+                                                    cn_get_all,
+                                                    update_from_cn,
+                                                    mock_get_by_host):
         svc_get_by_binary.return_value = [objects.Service(host='fake'),
                                           objects.Service(host='other')]
         cn_get_all.return_value = [
@@ -658,8 +716,13 @@ class HostManagerTestCase(test.NoDBTestCase):
             set, {'other': set([1])})
         self.host_manager.aggs_by_id = {1: fake_agg}
 
-        hosts = self.host_manager.get_all_host_states('fake-context')
-        # get_all_host_states returns a generator, so make a map from it
+        context = nova_context.get_admin_context()
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         host_state = host_states_map[('fake', 'fake')]
@@ -670,11 +733,10 @@ class HostManagerTestCase(test.NoDBTestCase):
     @mock.patch.object(host_manager.HostState, '_update_from_compute_node')
     @mock.patch.object(objects.ComputeNodeList, 'get_all')
     @mock.patch.object(objects.ServiceList, 'get_by_binary')
-    def test_get_all_host_states_corrupt_aggregates_info(self,
-                                                         svc_get_by_binary,
-                                                         cn_get_all,
-                                                         update_from_cn,
-                                                         mock_get_by_host):
+    def test_get_host_states_corrupt_aggregates_info(self, svc_get_by_binary,
+                                                     cn_get_all,
+                                                     update_from_cn,
+                                                     mock_get_by_host):
         """Regression test for bug 1605804
 
         A host can be in multiple host-aggregates at the same time. When a
@@ -700,16 +762,14 @@ class HostManagerTestCase(test.NoDBTestCase):
         aggregate.hosts = [host_a]
         self.host_manager.delete_aggregate(aggregate)
 
-        self.host_manager.get_all_host_states('fake-context')
+        context = nova_context.get_admin_context()
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+
+        self.host_manager._get_host_states(context, compute_nodes, services)
 
-    @mock.patch('nova.objects.ServiceList.get_by_binary')
-    @mock.patch('nova.objects.ComputeNodeList.get_all')
     @mock.patch('nova.objects.InstanceList.get_by_host')
-    def test_get_all_host_states_updated(self, mock_get_by_host,
-                                         mock_get_all_comp,
-                                         mock_get_svc_by_binary):
-        mock_get_all_comp.return_value = fakes.COMPUTE_NODES
-        mock_get_svc_by_binary.return_value = fakes.SERVICES
+    def test_host_state_update(self, mock_get_by_host):
         context = 'fake_context'
         hm = self.host_manager
         inst1 = objects.Instance(uuid=uuids.instance)
@@ -725,14 +785,8 @@ class HostManagerTestCase(test.NoDBTestCase):
         self.assertTrue(host_state.instances)
         self.assertEqual(host_state.instances[uuids.instance], inst1)
 
-    @mock.patch('nova.objects.ServiceList.get_by_binary')
-    @mock.patch('nova.objects.ComputeNodeList.get_all')
     @mock.patch('nova.objects.InstanceList.get_uuids_by_host')
-    def test_get_all_host_states_not_updated(self, mock_get_by_host,
-                                             mock_get_all_comp,
-                                             mock_get_svc_by_binary):
-        mock_get_all_comp.return_value = fakes.COMPUTE_NODES
-        mock_get_svc_by_binary.return_value = fakes.SERVICES
+    def test_host_state_not_updated(self, mock_get_by_host):
         context = 'fake_context'
         hm = self.host_manager
         inst1 = objects.Instance(uuid=uuids.instance)
@@ -1061,24 +1115,28 @@ class HostManagerChangedNodesTestCase(test.NoDBTestCase):
     @mock.patch('nova.objects.ServiceList.get_by_binary')
     @mock.patch('nova.objects.ComputeNodeList.get_all')
     @mock.patch('nova.objects.InstanceList.get_uuids_by_host')
-    def test_get_all_host_states(self, mock_get_by_host, mock_get_all,
-                                 mock_get_by_binary):
+    def test_get_host_states(self, mock_get_by_host, mock_get_all,
+                             mock_get_by_binary):
         mock_get_by_host.return_value = []
         mock_get_all.return_value = fakes.COMPUTE_NODES
         mock_get_by_binary.return_value = fakes.SERVICES
         context = 'fake_context'
 
-        # get_all_host_states returns a generator, so make a map from it
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
-                           self.host_manager.get_all_host_states(context)}
+                           self.host_manager._get_host_states(
+                               context, compute_nodes, services)}
         self.assertEqual(len(host_states_map), 4)
 
     @mock.patch('nova.objects.ServiceList.get_by_binary')
     @mock.patch('nova.objects.ComputeNodeList.get_all')
     @mock.patch('nova.objects.InstanceList.get_uuids_by_host')
-    def test_get_all_host_states_after_delete_one(self, mock_get_by_host,
-                                                  mock_get_all,
-                                                  mock_get_by_binary):
+    def test_get_host_states_after_delete_one(self, mock_get_by_host,
+                                              mock_get_all,
+                                              mock_get_by_binary):
         getter = (lambda n: n.hypervisor_hostname
                   if 'hypervisor_hostname' in n else None)
         running_nodes = [n for n in fakes.COMPUTE_NODES
@@ -1090,14 +1148,20 @@ class HostManagerChangedNodesTestCase(test.NoDBTestCase):
         context = 'fake_context'
 
         # first call: all nodes
-        hosts = self.host_manager.get_all_host_states(context)
-        # get_all_host_states returns a generator, so make a map from it
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         self.assertEqual(len(host_states_map), 4)
 
         # second call: just running nodes
-        hosts = self.host_manager.get_all_host_states(context)
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         self.assertEqual(len(host_states_map), 3)
@@ -1105,23 +1169,29 @@ class HostManagerChangedNodesTestCase(test.NoDBTestCase):
     @mock.patch('nova.objects.ServiceList.get_by_binary')
     @mock.patch('nova.objects.ComputeNodeList.get_all')
     @mock.patch('nova.objects.InstanceList.get_uuids_by_host')
-    def test_get_all_host_states_after_delete_all(self, mock_get_by_host,
-                                                  mock_get_all,
-                                                  mock_get_by_binary):
+    def test_get_host_states_after_delete_all(self, mock_get_by_host,
+                                              mock_get_all,
+                                              mock_get_by_binary):
         mock_get_by_host.return_value = []
         mock_get_all.side_effect = [fakes.COMPUTE_NODES, []]
         mock_get_by_binary.side_effect = [fakes.SERVICES, fakes.SERVICES]
         context = 'fake_context'
 
         # first call: all nodes
-        hosts = self.host_manager.get_all_host_states(context)
-        # get_all_host_states returns a generator, so make a map from it
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
+        # _get_host_states returns a generator, so make a map from it
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         self.assertEqual(len(host_states_map), 4)
 
         # second call: no nodes
-        hosts = self.host_manager.get_all_host_states(context)
+        compute_nodes, services = self.host_manager._get_computes_for_cells(
+            context, self.host_manager.enabled_cells)
+        hosts = self.host_manager._get_host_states(
+            context, compute_nodes, services)
         host_states_map = {(state.host, state.nodename): state for state in
                            hosts}
         self.assertEqual(len(host_states_map), 0)
@@ -1161,7 +1231,7 @@ class HostStateTestCase(test.NoDBTestCase):
     """Test case for HostState class."""
 
     # update_from_compute_node() and consume_from_request() are tested
-    # in HostManagerTestCase.test_get_all_host_states()
+    # in HostManagerTestCase.test_get_host_states()
 
     @mock.patch('nova.utils.synchronized',
                 side_effect=lambda a: lambda f: lambda *args: f(*args))
